/**************************************************************************************************
  CoreTimer.c
  Core Timer driver
**************************************************************************************************/

// Include Files
#include "__Time.h"
#include "CoreTimer.h"
#include "UART1.h"
//#include "timers.h"



// Variables
unsigned short int TimerFlag = 0;  // Set by interrupt
unsigned long int msec = 0;   // Millisecond counter (0 - 999)
long Epoch;  // Number of seconds since January 1, 1970 0h00mn00s
TimeStruct ts1;


/*************************************************************************************************/
void CoreTimer_Interrupt() iv IVT_CORE_TIMER ilevel 7 ics ICS_SRS {
     unsigned long int RegValue;

     msec++;
     if (msec>999) {                   // Check for overflow
         msec = 0;
         Epoch += 1;                   // Increment Date/Time by 1 second
         TimerFlag = 1;                // One second flag
     
         //Net_Ethernet_Intern_UserTimerSec++;  // Provide 1 second clock for ethernet intern
     }

     RegValue = CP0_GET(CP0_COMPARE);  // Get Core Timer current value
     RegValue += 40000;                // Increment value by 1 ms
     CP0_SET(CP0_COMPARE, RegValue);   // Set Compare register to new value
     CTIF_bit = 0;                     // Clear Core Timer interrupt flag
}

/*************************************************************************************************/
void InitCoreTimer(void) {
     ts1.ss = 0;     // Cold Start values
     ts1.mn = 0;
     ts1.hh = 0;
     ts1.md = 28;
     ts1.wd = 3;
     ts1.mo = 6;
     ts1.yy = 2012;

     Epoch = Time_dateToEpoch(&ts1);

     CP0_SET(CP0_COUNT, 0);        // Start counter at 0
     CP0_SET(CP0_COMPARE, 40000);  // Compare in 1ms

     CTIP0_bit = 1;  // Set Core Timer priority
     CTIP1_bit = 1;
     CTIP2_bit = 1;

     CTIS0_bit = 1;  // Set Core Timer sub-priority
     CTIS1_bit = 1;

     CTIF_bit = 0;  // Clear Core Timer interrupt flag
     CTIE_bit = 1;  // Enable Core Timer interrupts
}

/*************************************************************************************************/
void TimerCheck() {
     if (TimerFlag == 1) {
         TimerFlag = 0;  // Reset timer flag
         Time_epochToDate(Epoch, &ts1);  // Update date/time values each second
         
         //Net_Ethernet_Intern_UserTimerSec++;
         //UART1_PutQueue("Second tick\r\n");
         /*irqhandler_timer();*/
     }
}