/**************************************************************************************************
  UART1.c
  Interrupt driven serial interface routines.
**************************************************************************************************/

#include "Common.h"
#include "UART1.h"
#include "Buffers.h"
#include "__Time.h"
#include "MMC_Driver.h"
#include "__Lib_MmcFat16.h"



// Variables
char RcvBuf[RcvBufSize];     // Final receive message is assembled here
char RcvBufPtr;              // Pointer to next character in receive buffer

char *XmtBuf;                // Pointer to the current message being transmitted
char XmtBufPtr;              // Pointer to next character in transmit message
char UART_Busy = FALSE;      // UART busy flag


rx_q_struct RxQ;             // Receive buffer structure
tx_q_struct TxQ;             // Transmit buffer structure

// The receive message routine has 2 states
tPointerFcn UART1_RxFuncPtr = UART1_Rx1;  // Start in State 1


//extern TimeStruct ts1;
//extern long Epoch;
//extern unsigned long int msec;

char FileDataBfr[512];
//unsigned int NoRead;
//extern char FileName[16];
int Counter1;
//short fhandle;

char Ch;

//extern tPointerFcn AppFuncPtr;
//extern char ReadFileName[16];
//char FileRdBuf[128];
extern char LineBuf[128];

/*************************************************************************************************/
// Hardware Level Functions
/**************************************************************************************************
  This is the UART1 Interrupt Handler. */
void UART1_Interrupt() iv IVT_UART_1 ilevel 7 ics ICS_AUTO {
     char R_data;        // Receive data
     char T_data;        // Transmit data
   
     if (URXDA_bit) {
         R_data = U1RXREG;              // Fetch a byte from receive register

         RxQ.RxQueue[RxQ.RxIn++] = R_data;      // Put it into the recirculating buffer
         if (RxQ.RxIn > (rxqsize-1))    // Adjust pointer if necessary
             RxQ.RxIn = 0;
         U1RXIF_bit = 0;                // Clear the receive interrupt flag

     } else {
         T_data = XmtBuf[XmtBufPtr++];  // Get a byte from the transmit buffer
         if (T_data == NUL) {           // If it is the end of message marker
             U1TXIE_bit = 0;            // Disable transmitter interrupts
         } else {
             U1TXREG = T_data;          // Transfer data byte to transmit register
             U1TXIF_bit = 0;            // Clear transmit interrupt flag
         }
     }
}

/**************************************************************************************************
  This routine sets UART1 baud rate. */
void UART1_SetBaud(void) {
     //U1BRG = UxBRG;      // Calculates a value of 519. 520 is closer to 9600 baud
     U1BRG = 520;
}

/**************************************************************************************************
  This routine starts the UART1 receiver. */
void UART1_StartRx(void) {
     ON__U1MODE_bit = 1; // Enable UART1 circuit
     URXEN_bit = 1;      // Enable receiver

     U1IP0_bit = 0;      // Set UART1 priority
     U1IP1_bit = 1;
     U1IP2_bit = 1;
     U1IS1_bit = 1;      // Set UART1 sub-priority
     U1IS0_bit = 1;
     
     URXISEL0_bit = 0;   // Interrupt when buffer is not empty
     URXISEL1_bit = 0;

     U1RXIF_bit = 0;     // Clear the receive interrupt flag
     U1RXIE_bit = 1;     // Enable receiver interrupts
}

/**************************************************************************************************
  This routine starts the UART1 transmitter */
void UART1_StartTx(void) {
     UTXEN_bit = 1;      // Enable transmitter

     UTXISEL0_bit = 0;   // Interrupt when transmit buffer is empty
     UTXISEL1_bit = 1;

     U1TXIF_bit = 0;     // Clear transmit interrupt flag
     U1TXIE_bit = 1;     // Enable transmitter interrupts
}




/*************************************************************************************************/
// UART1 Functions
/**************************************************************************************************
  Initialize UART1 */
void InitUART1(void) {
     UART1_SetBaud();
     UART1_RxQueueInit();
     UART1_TxQueueInit();
     UART1_StartRx();
}

/**************************************************************************************************
  This routine initializes the transmit buffer queue in and out pointers */
void UART1_TxQueueInit(void) {
     TxQ.TxIn = 0;
     TxQ.TxOut = 0;
     XmtBufPtr = 0;
}

/**************************************************************************************************
  This routine initializes the receive buffer in and out pointers. */
void UART1_RxQueueInit(void) {
     RxQ.RxIn = 0;
     RxQ.RxOut = 0;
     RcvBufPtr = 0;
}

/**************************************************************************************************
  This routine enters a pointer to a text message and starts the UART1 transmitter. */
void UART1_SendMsg(char *s) {
     XmtBuf = s;           // Pointer to the message for the UART1 interrupt handler
     XmtBufPtr = 0;        // Set character counter to start of text message
     UART1_StartTx();      // Turn on UART1 transmitter
}

/**************************************************************************************************
  This routine checks the transmit buffer queue and if it is not empty, it extracts the pointer
  to the message and sets up the UART1 transmit interrupt handler. */

// Call this function in main loop
void UART1_CheckQueue(void) {
     
     // Don't try to start transmitter if already transmitting
     if ((!U1TXIE_bit)&&(TRMT_bit)) {

         if (TxQ.TxIn != TxQ.TxOut) {       // If pointers are not the same
             UART1_SendMsg(TxQ.TxQueue[TxQ.TxOut++]);  // Hand off the message pointer and increment counter
             if (TxQ.TxOut > (txqsize-1))   // Check for wrap
                 TxQ.TxOut = 0;
         } else {
             UART_Busy = FALSE;
         }
     }
}

/**************************************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates the in pointer to
  the next available location. */
void UART1_PutQueue(char *msg) {
     TxQ.TxQueue[TxQ.TxIn++] = msg;      // Enter the message pointer into the queue
     if (TxQ.TxIn > (txqsize-1))         // Check for wrap
         TxQ.TxIn = 0;

     UART_Busy = TRUE;
}

/**************************************************************************************************
  This routine retrieves a byte from the receive buffer if present. Returns 0 if no data available.
  Returns 1 if a data byte has been transferred successfully. The pointer to the data byte is
  passed to the routine. */
char UART1_GetChar(char *rxbyte) {
     if (RxQ.RxIn != RxQ.RxOut){             // If pointers are not the same
         *rxbyte = RxQ.RxQueue[RxQ.RxOut++];  // Fetch a character from the recirculating buffer
         if (RxQ.RxOut > (rxqsize-1))         // Check for wrap
             RxQ.RxOut = 0;
         return 1;
     } else {
         return 0;
     }
}

/**************************************************************************************************
  UART1_Rx State Machine. Call this function in main loop */
void UART1_Rx(void) {
     UART1_RxFuncPtr();        // Use the function pointer to go to a state
}

/**************************************************************************************************
  UART1 State 1: Accumulate received characters in a buffer until a Line Feed character is
  received then insert a NUL character into the buffer and go to State 2.*/
void UART1_Rx1(void) {
     char rxchar;

     if (UART1_GetChar(&rxchar)) {
         switch (rxchar){
            case NUL:     // Discard NUL character
                break;

            case '\r':    // Discard return character
                break;

            case '\n':    // New line character
                RcvBuf[RcvBufPtr++] = rxchar;     // Put received character into buffer
                RcvBuf[RcvBufPtr] = 0;            // End marker
                RcvBufPtr = 0;                    // Point to start of buffer
                UART1_RxFuncPtr = UART1_Rx2;      // Go to state 2
                break;

            default:
               if (RcvBufPtr < (RcvBufSize-2))   // Prevents buffer overrun
                   RcvBuf[RcvBufPtr++] = rxchar;  // Put character in buffer
               break;
         }
     }
}

/**************************************************************************************************
  UART1 State 2: Execute the received command and go to State 1*/
void UART1_Rx2(void) {
     char *BufNum;

     switch (RcvBuf[0]) {

         // Change to suit application

         case 'a':
             UART1_PutQueue("ACK\r\n");
             break;

         case 'b':
             break;

         case 'c':
             break;

         case 'd':
             break;
             
         case 'e':
             break;

         case 'f':
             break;

         case 'g':
             break;

         case 'h':
             break;
             
         case 'i':
             break;
         
         case 'j':
             break;
         
          case 'k':
             break;

         case 'l':
             break;
         
         case 'm':
             break;
         
         case 'n':
             break;
         
         case 'o':
             break;

         case 'p':
             break;
         
         case 'q':
             break;
         
         case 'r':
             break;
         
         case 's':
             break;
         
         case 't':
             break;
         
         case 'u':
             break;
         
         case 'v':
             break;
         
         case 'w':
             if (OpenFile("Test1.txt")) {
                 
                 while (ReadFile(FileDataBfr) == 0) {
                     UART1_PutQueue(FileDataBfr);
                     
                     while (UART_Busy == TRUE) {
                         UART1_CheckQueue();
                     }
                 }
                 
                 CloseFile();
             }
             break;
         
         case 'x':
             break;
         
         case 'y':
             Counter1 = 0;
             memset(FileDataBfr,0,512);  // Clear the buffer
             Mmc_Fat_Dir(PrintChar);
             strcat(FileDataBfr,"\r\n");
             UART1_PutQueue(FileDataBfr);
             break;
         
         
         // Add more commands as needed
             


         
         
         default:
             UART1_PutQueue("Unknown command: ");
           
             BufNum = GetBuffer();   // Get a free buffer
             (void)sprintf(BufNum,"%c\r\n",RcvBuf[0]);
             UART1_PutQueue(BufNum);
             break;
     }
     UART1_RxFuncPtr = UART1_Rx1;    // Go back to state 1
}

void PrintChar(char Ch) {
     FileDataBfr[Counter1++] = Ch;
}