/******************************************************************************
  UART1.h
 *****************************************************************************/
#ifndef UART1_H
#define UART1_H

#define Fpb 80000000
#define UART1_BAUD_RATE 9600  // Desired baud rate
#define UxBRG (Fpb/(16*UART1_BAUD_RATE)-1) // Generates 519 (520 is closer)

#define RcvBufSize 64    // Number of bytes in Receive Buffer
#define rxqsize 64       // Number of characters in receive buffer
#define txqsize 32       // Number of pointers in transmit queue
#define DC3 0x13         // XOFF control character
#define DC1 0x11         // XON control character
#define NUL 0x00         // NUL control character


//#define DEBUG 1          // Set to 0 to prevent diagnostic messages

//#if DEBUG == 1
//#define DEBUGOUT(c) UART1_PutQueue(c)
//#else
//#define DEBUGOUT(c) {}
//#endif


 
typedef void (*tPointerFcn)(void);  // Pointer to a function
typedef char *pnt;
 
//Rx buffer and the input and output pointers
 typedef struct rx_q_struct
 {
     int RxIn;               // Input pointer
     int RxOut;              // Output pointer
     char RxQueue[rxqsize];  // Array of characters
 } rx_q_struct;
 
// Tx buffer and its input and output pointers
typedef struct tx_q_struct
{
     int TxIn;               // Input pointer
     int TxOut;              // Output pointer
     pnt TxQueue[txqsize];   // Array of pointers
} tx_q_struct;



// Function Prototypes
void UART1_SetBaud(void);
void UART1_StartRx(void);
void UART1_StartTx(void);

void InitUART1(void);
void UART1_TxQueueInit(void);
void UART1_RxQueueInit(void);

void UART1_SendMsg(char *s);
void UART1_CheckQueue(void);
void UART1_PutQueue(char *msg);
char UART1_GetChar(char *rxbyte);

// State Machine
void UART1_Rx(void);
void UART1_Rx1(void);
void UART1_Rx2(void);

void PrintChar(char Ch);

#endif